@@ -1,7 +1,22 @@
-intercept<-function (ogg, parm, gap=TRUE, rev.sgn = FALSE, var.diff = FALSE){
-#E' come sotto ma unisce due cicli for..
-#corregge in caso di no model intercept
-#gap: mostra le interc con gap ..
+intercept<-function (ogg, parm, gap=TRUE, rev.sgn = FALSE, var.diff = FALSE, 
+    digits = max(3, getOption("digits") - 3)){
+#corregge in caso di no model intercept -- CHE VOLEVO DIRE??
+#--
+        f.U<-function(nomiU, term=NULL){
+        #trasforma i nomi dei coeff U (o V) nei nomi delle variabili corrispondenti
+        #and if 'term' is provided (i.e. it differs from NULL) the index of nomiU matching term are returned
+            k<-length(nomiU)
+            nomiUsenzaU<-strsplit(nomiU, "\\.")
+            nomiU.ok<-vector(length=k)
+            for(i in 1:k){
+                nomi.i<-nomiUsenzaU[[i]][-1]
+                if(length(nomi.i)>1) nomi.i<-paste(nomi.i,collapse=".")
+                nomiU.ok[i]<-nomi.i
+                }
+          if(!is.null(term)) nomiU.ok<-(1:k)[nomiU.ok%in%term]
+          return(nomiU.ok)
+        }
+#--        
     if (!"segmented" %in% class(ogg))
         stop("A segmented model is needed")
     if (var.diff && length(ogg$nameUV$Z) > 1) {
@@ -32,16 +47,15 @@ intercept<-function (ogg, parm, gap=TRUE, rev.sgn = FALSE, var.diff = FALSE){
     Allpsi <- index <- vector(mode = "list", length = length(nomeZ))
     gapCoef<-summary.segmented(ogg)$gap
     Ris <- list()
-    digits <- max(3, getOption("digits") - 3)
     rev.sgn <- rep(rev.sgn, length.out = length(nomeZ))
     if("(Intercept)"%in%names(coef(ogg))){
         alpha0 <- alpha00 <- coef(ogg)["(Intercept)"]} else {alpha0 <- alpha00 <-0}
 #per ogni variabile segmented...
     for (i in 1:length(nomeZ)) {
-        id.cof.U <- grep(paste("\\.", nomeZ[i], "$", sep = ""),
-            nomi, value = FALSE)
-        psii <- ogg$psi[grep(paste("\\.", nomeZ[i], "$", sep = ""),
-            rownames(ogg$psi), value = FALSE), 2]
+#        id.cof.U <- grep(paste("\\.", nomeZ[i], "$", sep = ""), nomi, value = FALSE)
+#        psii <- ogg$psi[grep(paste("\\.", nomeZ[i], "$", sep = ""), rownames(ogg$psi), value = FALSE), 2]
+        id.cof.U <- f.U(ogg$nameUV$U, nomeZ[i]) + (match(ogg$nameUV$U[1], nomi)-1)
+        psii<- ogg$psi[f.U(ogg$nameUV$V, nomeZ[i]) , "Est."] 
         Allpsi[[i]] <- sort(psii, decreasing = FALSE) 
         id.cof.U <- id.cof.U[order(psii)]
         index[[i]] <- id.cof.U
@@ -49,7 +63,8 @@ intercept<-function (ogg, parm, gap=TRUE, rev.sgn = FALSE, var.diff = FALSE){
         ind <- as.numeric(na.omit(unlist(index[[i]])))
         cof <- coef(ogg)[ind]
         alpha <- vector(length = length(ind))
-        gapCoef.i<-gapCoef[grep(paste("\\.",nomeZ[i],"$",sep=""), rownames(gapCoef), value = FALSE),"Est."]
+        #gapCoef.i<-gapCoef[grep(paste("\\.",nomeZ[i],"$",sep=""), rownames(gapCoef), value = FALSE),"Est."]
+        gapCoef.i<-gapCoef[f.U(rownames(gapCoef), nomeZ[i]) ,"Est."]
         for (j in 1:length(cof)) {
             alpha[j] <- alpha0 - Allpsi[[i]][j] * cof[j]
             if(gap) alpha[j] <- alpha[j] - gapCoef.i[j]
@@ -59,8 +74,7 @@ intercept<-function (ogg, parm, gap=TRUE, rev.sgn = FALSE, var.diff = FALSE){
         #if(gap) alpha<-alpha -gapCoef[grep(paste("\\.",nomeZ[i],"$",sep=""), rownames(gapCoef), value = FALSE),"Est."]
         cof.out <- c(alpha00, alpha)
         ris <- matrix(cof.out)
-        dimnames(ris) <- list(paste("intercept", 1:nrow(ris),
-            sep = ""), "Est.")
+        dimnames(ris) <- list(paste("intercept", 1:nrow(ris), sep = ""), "Est.")
         Ris[[nomeZ[i]]] <- signif(ris, digits)
     }
     Ris
