@@ -1,19 +1,36 @@
 `slope` <-
-function(ogg, parm, conf.level=0.95, rev.sgn=FALSE, var.diff=FALSE, APC=FALSE){
+function(ogg, parm, conf.level=0.95, rev.sgn=FALSE, var.diff=FALSE, APC=FALSE,
+      digits = max(3, getOption("digits") - 3)){
+#--
+        f.U<-function(nomiU, term=NULL){
+        #trasforma i nomi dei coeff U (o V) nei nomi delle variabili corrispondenti
+        #and if 'term' is provided (i.e. it differs from NULL) the index of nomiU matching term are returned
+            k<-length(nomiU)
+            nomiUsenzaU<-strsplit(nomiU, "\\.")
+            nomiU.ok<-vector(length=k)
+            for(i in 1:k){
+                nomi.i<-nomiUsenzaU[[i]][-1]
+                if(length(nomi.i)>1) nomi.i<-paste(nomi.i,collapse=".")
+                nomiU.ok[i]<-nomi.i
+                }
+          if(!is.null(term)) nomiU.ok<-(1:k)[nomiU.ok%in%term]
+          return(nomiU.ok)
+        }
+#--        
         if(!"segmented"%in%class(ogg)) stop("A segmented model is needed")
         if(var.diff && length(ogg$nameUV$Z)>1) {
             var.diff<-FALSE
             warning("var.diff set to FALSE with multiple segmented variables", call.=FALSE)
             }
         nomepsi<-rownames(ogg$psi) #OK
-        nomeU<-ogg$nameUV[[1]]
-        nomeZ<-ogg$nameUV[[3]]
+        nomeU<-ogg$nameUV$U
+        nomeZ<-ogg$nameUV$Z
         if(missing(parm)) {
           nomeZ<- ogg$nameUV[[3]]
           if(length(rev.sgn)==1) rev.sgn<-rep(rev.sgn,length(nomeZ))
           }
              else {
-                if(! all(parm %in% ogg$nameUV[[3]])) {stop("invalid parm")}
+                if(! all(parm %in% ogg$nameUV$Z)) {stop("invalid parm")}
                   else {nomeZ<-parm}
                   }
         if(length(rev.sgn)!=length(nomeZ)) rev.sgn<-rep(rev.sgn, length.out=length(nomeZ))
@@ -23,8 +40,14 @@ function(ogg, parm, conf.level=0.95, rev.sgn=FALSE, var.diff=FALSE, APC=FALSE){
         for(i in 1:length(nomeZ)) {
             #id.cof.U<-grep(paste("\\.",nomeZ[i],"$",sep=""), nomi, value=FALSE)
             #psii<-ogg$psi[grep(paste("\\.",nomeZ[i],"$",sep=""), rownames(ogg$psi), value=FALSE),2]
-            id.cof.U<- match(grep(nomeZ[i],   ogg$nameUV$U, value=TRUE), nomi)
-            psii<-ogg$psi[grep(nomeZ[i],   ogg$nameUV$V, value=TRUE),2]
+            #id.cof.U<- match(grep(nomeZ[i],   ogg$nameUV$U, value=TRUE), nomi)
+            #psii<-ogg$psi[grep(nomeZ[i],   ogg$nameUV$V, value=TRUE),2]
+            #il paste con "$" (paste("\\.",nomeZ[i],"$",sep=""))  utile perch serve a distinguere variabili con nomi simili (ad es., "x" e "xx")
+            #Comunque nella versione dopo la 0.3-1.0 ho (FINALMENTE) risolto mettendo f.U
+            id.cof.U<- f.U(ogg$nameUV$U, nomeZ[i])
+            #id.cof.U  la posizione nel vettore ogg$nameUV$U; la seguente corregge per eventuali variabili che ci sono prima (ad es., interc)
+            id.cof.U<- id.cof.U + (match(ogg$nameUV$U[1], nomi)-1)            
+            psii<- ogg$psi[f.U(ogg$nameUV$V, nomeZ[i]) , "Est."]
             id.cof.U <- id.cof.U[order(psii)]            
             index[[i]]<-c(match(nomeZ[i],nomi), id.cof.U)
             }
@@ -52,7 +75,8 @@ function(ogg, parm, conf.level=0.95, rev.sgn=FALSE, var.diff=FALSE, APC=FALSE){
             ris<-cbind(cof.out,se.out,(cof.out/se.out),(cof.out-k),(cof.out+k))
             cin<-paste("CI","(",conf.level*100,"%",")",c(".l",".u"),sep="")
             #se la left slope  nulla....
-            if(identical(length(ind),length(grep(paste("\\.",nomeZ[i],"$",sep=""), nomeU)))){
+            #if(identical(length(ind),length(grep(paste("\\.",nomeZ[i],"$",sep=""), nomeU)))){
+            if(!nomeZ[i]%in%nomi){            
                     ris<-rbind(c(0,rep(NA,(ncol(ris)-1))),ris)}
             if(rev.sgn[i]){
                 ris<-cbind(-ris[nrow(ris):1,1],ris[nrow(ris):1,2],-ris[nrow(ris):1,3],
